@@ -4,6 +4,7 @@ use serde::{Deserialize, Serialize};
 use ockam_vault::PublicKey;
 
 use crate::change_history::ProfileChangeHistory;
+use crate::profile::Profile;
 use crate::{EventIdentifier, KeyAttributes, ProfileChangeEvent, ProfileIdentifier, ProfileVault};
 use ockam_core::{allow, deny};
 
@@ -106,4 +107,8 @@ impl Contact {
     pub fn get_last_event_id(&self) -> ockam_core::Result<EventIdentifier> {
         self.change_history.get_last_event_id()
     }
+    /// Get BBS+ signing public key
+    pub fn get_signing_public_key(&self) -> ockam_core::Result<PublicKey> {
+        self.get_public_key(&KeyAttributes::new(Profile::CREDENTIALS_ISSUE.to_string()))
+    }
 }
