@@ -1,13 +1,15 @@
 use crate::EntityError::IdentityApiFailed;
 use crate::{
-    profile::Profile, traits::Verifier, worker::EntityWorker, AuthenticationProof, Changes,
-    Contact, Credential, CredentialAttribute, CredentialFragment1, CredentialFragment2,
-    CredentialOffer, CredentialPresentation, CredentialProof, CredentialPublicKey,
-    CredentialRequest, CredentialRequestFragment, CredentialSchema, Handle, Holder, Identity,
-    IdentityRequest, IdentityResponse, Issuer, MaybeContact, OfferId, PresentationManifest,
-    ProfileChangeEvent, ProfileIdentifier, ProofRequestId, SecureChannels, SigningPublicKey,
-    TrustPolicy, TrustPolicyImpl,
+    profile::Profile, traits::Verifier, worker::EntityWorker, AuthenticationProof, BbsCredential,
+    Changes, Contact, Credential, CredentialAttribute, CredentialFragment1, CredentialFragment2,
+    CredentialOffer, CredentialPresentation, CredentialProof, CredentialProtocol,
+    CredentialPublicKey, CredentialRequest, CredentialRequestFragment, CredentialSchema,
+    EntityCredential, Handle, Holder, HolderWorker, Identity, IdentityRequest, IdentityResponse,
+    Issuer, ListenerWorker, MaybeContact, OfferId, PresentationFinishedMessage,
+    PresentationManifest, PresenterWorker, ProfileChangeEvent, ProfileIdentifier, ProofRequestId,
+    SecureChannels, SigningPublicKey, TrustPolicy, TrustPolicyImpl, VerifierWorker,
 };
+use ockam_core::lib::convert::TryInto;
 use ockam_core::{Address, Result, Route};
 use ockam_node::{block_future, Context};
 use ockam_vault::ockam_vault_core::{PublicKey, Secret};
@@ -22,10 +24,10 @@ pub struct Entity {
 }
 
 impl Entity {
-    pub fn new(handle: Handle, profile_id: ProfileIdentifier) -> Self {
+    pub fn new(handle: Handle, profile_id: Option<ProfileIdentifier>) -> Self {
         Entity {
             handle,
-            current_profile_id: Some(profile_id),
+            current_profile_id: profile_id,
         }
     }
 
@@ -33,18 +35,15 @@ impl Entity {
         self.handle.clone()
     }
 
-    pub fn create(node_ctx: &Context) -> Result<Entity> {
-        block_future(&node_ctx.runtime(), async move {
-            let ctx = node_ctx.new_context(Address::random(0)).await?;
+    pub fn create(ctx: &Context, vault_address: &Address) -> Result<Entity> {
+        block_future(&ctx.runtime(), async move {
+            let ctx = ctx.new_context(Address::random(0)).await?;
             let address = Address::random(0);
             ctx.start_worker(&address, EntityWorker::default()).await?;
 
-            let mut entity = Entity {
-                handle: Handle::new(ctx, address),
-                current_profile_id: None,
-            };
+            let mut entity = Entity::new(Handle::new(ctx, address), None);
 
-            let default_profile = entity.create_profile()?;
+            let default_profile = entity.create_profile(vault_address)?;
             entity.current_profile_id = Some(default_profile.identifier()?);
             Ok(entity)
         })
@@ -70,8 +69,8 @@ fn err<T>() -> Result<T> {
 }
 
 impl Entity {
-    pub fn create_profile(&mut self) -> Result<Profile> {
-        if let Res::CreateProfile(id) = self.call(CreateProfile)? {
+    pub fn create_profile(&mut self, vault_address: &Address) -> Result<Profile> {
+        if let Res::CreateProfile(id) = self.call(CreateProfile(vault_address.clone()))? {
             Ok(Profile::new(id, self.handle.clone()))
         } else {
             err()
@@ -251,7 +250,7 @@ impl SecureChannels for Entity {
         let profile = self.current_profile().expect("no current profile");
         let ctx = &self.handle().ctx;
         let trust_policy_address = block_future(&self.handle.ctx.runtime(), async move {
-            TrustPolicyImpl::create(ctx, trust_policy).await
+            TrustPolicyImpl::create_worker(ctx, trust_policy).await
         })?;
         if let Res::CreateSecureChannelListener = self.call(CreateSecureChannelListener(
             profile.identifier().expect("couldn't get profile id"),
@@ -272,7 +271,7 @@ impl SecureChannels for Entity {
         let profile = self.current_profile().expect("no current profile");
         let ctx = &self.handle().ctx;
         let trust_policy_address = block_future(&self.handle.ctx.runtime(), async move {
-            TrustPolicyImpl::create(ctx, trust_policy).await
+            TrustPolicyImpl::create_worker(ctx, trust_policy).await
         })?;
         if let Res::CreateSecureChannel(address) = self.call(CreateSecureChannel(
             profile.identifier().expect("couldn't get profile id"),
@@ -337,7 +336,7 @@ impl Issuer for Entity {
         &self,
         schema: &CredentialSchema,
         attributes: A,
-    ) -> Result<Credential> {
+    ) -> Result<BbsCredential> {
         let profile = self.clone().current_profile().expect("no current profile");
         if let Res::SignCredential(credential) = profile.call(SignCredential(
             profile.identifier().expect("couldn't get profile id"),
@@ -396,7 +395,7 @@ impl Holder for Entity {
         &self,
         credential_fragment1: CredentialFragment1,
         credential_fragment2: CredentialFragment2,
-    ) -> Result<Credential> {
+    ) -> Result<BbsCredential> {
         let profile = self.clone().current_profile().expect("no current profile");
         if let Res::CombineCredentialFragments(credential) =
             profile.call(CombineCredentialFragments(
@@ -413,7 +412,7 @@ impl Holder for Entity {
 
     fn is_valid_credential(
         &self,
-        credential: &Credential,
+        credential: &BbsCredential,
         verifier_key: SigningPublicKey,
     ) -> Result<bool> {
         let profile = self.clone().current_profile().expect("no current profile");
@@ -428,9 +427,9 @@ impl Holder for Entity {
         }
     }
 
-    fn present_credential(
+    fn create_credential_presentation(
         &self,
-        credential: &Credential,
+        credential: &BbsCredential,
         presentation_manifests: &PresentationManifest,
         proof_request_id: ProofRequestId,
     ) -> Result<CredentialPresentation> {
@@ -446,6 +445,30 @@ impl Holder for Entity {
             err()
         }
     }
+
+    fn add_credential(&mut self, credential: EntityCredential) -> Result<()> {
+        let profile = self.clone().current_profile().expect("no current profile");
+        if let Res::AddCredential = profile.call(AddCredential(
+            profile.identifier().expect("couldn't get profile id"),
+            credential,
+        ))? {
+            Ok(())
+        } else {
+            err()
+        }
+    }
+
+    fn get_credential(&mut self, credential: &Credential) -> Result<EntityCredential> {
+        let profile = self.clone().current_profile().expect("no current profile");
+        if let Res::GetCredential(c) = profile.call(GetCredential(
+            profile.identifier().expect("couldn't get profile id"),
+            credential.clone(),
+        ))? {
+            Ok(c)
+        } else {
+            err()
+        }
+    }
 }
 
 impl Verifier for Entity {
@@ -499,3 +522,118 @@ impl Verifier for Entity {
         }
     }
 }
+
+impl CredentialProtocol for Entity {
+    fn create_credential_issuance_listener(
+        &mut self,
+        address: impl Into<Address> + Send,
+        schema: CredentialSchema,
+        trust_policy: impl TrustPolicy,
+    ) -> Result<()> {
+        let profile = self.clone().current_profile().expect("no current profile");
+        block_future(&self.handle.ctx.runtime(), async move {
+            let trust_policy =
+                TrustPolicyImpl::create_using_impl(&self.handle.ctx, trust_policy).await?;
+
+            let address = address.into();
+            let worker = ListenerWorker::new(profile, schema, trust_policy);
+            self.handle.ctx.start_worker(address, worker).await?;
+
+            Ok(())
+        })
+    }
+
+    fn acquire_credential(
+        &mut self,
+        issuer_route: Route,
+        issuer_id: &ProfileIdentifier,
+        schema: CredentialSchema,
+        values: Vec<CredentialAttribute>,
+    ) -> Result<Credential> {
+        let profile = self.clone().current_profile().expect("no current profile");
+        block_future(&self.handle.ctx.runtime(), async move {
+            let mut ctx = self.handle.ctx.new_context(Address::random(0)).await?;
+
+            let worker = HolderWorker::new(
+                profile.clone(),
+                issuer_id.clone(),
+                issuer_route,
+                schema,
+                values,
+                ctx.address(),
+            );
+            ctx.start_worker(Address::random(0), worker).await?;
+
+            let credential = ctx
+                .receive_timeout::<Credential>(120 /* FIXME */)
+                .await?
+                .take()
+                .body();
+
+            Ok(credential)
+        })
+    }
+
+    fn present_credential(
+        &mut self,
+        verifier_route: Route,
+        credential: Credential,
+        reveal_attributes: Vec<String>,
+    ) -> Result<()> {
+        let profile = self.clone().current_profile().expect("no current profile");
+        block_future(&self.handle.ctx.runtime(), async move {
+            let credential = self.get_credential(&credential)?;
+
+            let mut ctx = self.handle.ctx.new_context(Address::random(0)).await?;
+            let worker = PresenterWorker::new(
+                profile.clone(),
+                verifier_route,
+                credential,
+                reveal_attributes,
+                ctx.address(),
+            );
+            ctx.start_worker(Address::random(0), worker).await?;
+
+            let _ = ctx
+                .receive_timeout::<PresentationFinishedMessage>(120 /* FIXME */)
+                .await?
+                .take()
+                .body();
+
+            Ok(())
+        })
+    }
+
+    fn verify_credential(
+        &mut self,
+        address: impl Into<Address> + Send,
+        issuer_id: &ProfileIdentifier,
+        schema: CredentialSchema,
+        attributes_values: Vec<CredentialAttribute>,
+    ) -> Result<bool> {
+        let mut profile = self.clone().current_profile().expect("no current profile");
+        block_future(&self.handle.ctx.runtime(), async move {
+            let issuer = profile.get_contact(issuer_id)?.unwrap();
+            let pubkey = issuer.get_signing_public_key()?;
+
+            let mut ctx = self.handle.ctx.new_context(Address::random(0)).await?;
+            let worker = VerifierWorker::new(
+                profile.clone(),
+                pubkey.as_ref().try_into().unwrap(), // FIXME
+                schema,
+                attributes_values,
+                ctx.address(),
+            );
+
+            ctx.start_worker(address.into(), worker).await?;
+
+            let res = ctx
+                .receive_timeout::<bool>(120 /* FIXME */)
+                .await?
+                .take()
+                .body();
+
+            Ok(res)
+        })
+    }
+}
